@@ -1,7 +1,7 @@
 /*
- * tio - a simple TTY terminal I/O application
+ * tio - a simple serial terminal I/O tool
  *
- * Copyright (c) 2014-2018  Martin Lund
+ * Copyright (c) 2014-2022  Martin Lund
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,25 +30,29 @@
 #include <getopt.h>
 #include <termios.h>
 #include <limits.h>
-#include "tio/options.h"
-#include "tio/error.h"
+#include "options.h"
+#include "error.h"
+#include "misc.h"
+#include "print.h"
 
 /* Default options */
 struct option_t option =
 {
-    "",       // Device name
-    115200,   // Baudrate
-    8,        // Databits
-    "none",   // Flow
-    1,        // Stopbits
-    "none",   // Parity
-    0,        // No output delay
-    false,    // No autoconnect
-    false,    // No log
-    false,    // No local echo
-    false,    // No timestamp
-    "",       // Log filename
-    ""        // Map string
+    .tty_device = "",
+    .baudrate = 115200,
+    .databits = 8,
+    .flow = "none",
+    .stopbits = 1,
+    .parity = "none",
+    .output_delay = 0,
+    .no_autoconnect = false,
+    .log = false,
+    .local_echo = false,
+    .timestamp = TIMESTAMP_NONE,
+    .list_devices = false,
+    .log_filename = "",
+    .map = "",
+    .color = -1,
 };
 
 void print_help(char *argv[])
@@ -63,36 +67,89 @@ void print_help(char *argv[])
     printf("  -p, --parity odd|even|none  Parity (default: none)\n");
     printf("  -o, --output-delay <ms>     Output delay (default: 0)\n");
     printf("  -n, --no-autoconnect        Disable automatic connect\n");
-    printf("  -e, --local-echo            Do local echo\n");
-    printf("  -t, --timestamp             Prefix each new line with a timestamp\n");
-    printf("  -l, --log <filename>        Log to file\n");
+    printf("  -e, --local-echo            Enable local echo\n");
+    printf("  -t, --timestamp[=<format>]  Enable timestamp (default: 24hour)\n");
+    printf("  -L, --list-devices          List available serial devices\n");
+    printf("  -l, --log[=<filename>]      Log to file\n");
     printf("  -m, --map <flags>           Map special characters\n");
+    printf("  -c, --color <code>          Colorize tio text\n");
     printf("  -v, --version               Display version\n");
     printf("  -h, --help                  Display help\n");
     printf("\n");
-    printf("See the man page for list of supported mapping flags.\n");
+    printf("See the man page for more details.\n");
     printf("\n");
     printf("In session, press ctrl-t q to quit.\n");
     printf("\n");
 }
 
-long string_to_long(char *string)
+const char* timestamp_token(enum timestamp_t timestamp)
 {
-    long result;
-    char *end_token;
-
-    errno = 0;
-    result = strtol(string, &end_token, 10);
-    if ((errno != 0) || (*end_token != 0))
+    switch (timestamp)
     {
-        error_printf("Invalid digit");
-        exit(EXIT_FAILURE);
-    }
+        case TIMESTAMP_NONE:
+            return "none";
+            break;
 
-    return result;
+        case TIMESTAMP_24HOUR:
+            return "24hour";
+            break;
+
+        case TIMESTAMP_24HOUR_START:
+            return "24hour-start";
+            break;
+
+        case TIMESTAMP_ISO8601:
+            return "iso8601";
+            break;
+
+        default:
+            return "unknown";
+            break;
+    }
 }
 
-void parse_options(int argc, char *argv[])
+enum timestamp_t timestamp_option_parse(const char *arg)
+{
+    enum timestamp_t timestamp = TIMESTAMP_24HOUR; // Default
+    
+    if (arg != NULL)
+    {
+        if (strcmp(arg, "24hour-start") == 0)
+        {
+            return TIMESTAMP_24HOUR_START;
+        }
+        else if (strcmp(arg, "iso8601") == 0)
+        {
+            return TIMESTAMP_ISO8601;
+        }
+        else
+        {
+            printf("Warning: Unknown timestamp type, falling back to '24hour' default format\n");
+        }
+    }
+
+    return timestamp;
+}
+
+void options_print()
+{
+    tio_printf(" TTY device: %s", option.tty_device);
+    tio_printf(" Baudrate: %u", option.baudrate);
+    tio_printf(" Databits: %d", option.databits);
+    tio_printf(" Flow: %s", option.flow);
+    tio_printf(" Stopbits: %d", option.stopbits);
+    tio_printf(" Parity: %s", option.parity);
+    tio_printf(" Local echo: %s", option.local_echo ? "enabled" : "disabled");
+    tio_printf(" Timestamps: %s", timestamp_token(option.timestamp));
+    tio_printf(" Output delay: %d", option.output_delay);
+    tio_printf(" Auto connect: %s", option.no_autoconnect ? "disabled" : "enabled");
+    if (option.map[0] != 0)
+        tio_printf(" Map flags: %s", option.map);
+    if (option.log)
+        tio_printf(" Log file: %s", option.log_filename);
+}
+
+void options_parse(int argc, char *argv[])
 {
     int c;
 
@@ -114,9 +171,11 @@ void parse_options(int argc, char *argv[])
             {"output-delay",   required_argument, 0, 'o'},
             {"no-autoconnect", no_argument,       0, 'n'},
             {"local-echo",     no_argument,       0, 'e'},
-            {"timestamp",      no_argument,       0, 't'},
-            {"log",            required_argument, 0, 'l'},
+            {"timestamp",      optional_argument, 0, 't'},
+            {"list-devices",   no_argument,       0, 'L'},
+            {"log",            optional_argument, 0, 'l'},
             {"map",            required_argument, 0, 'm'},
+            {"color",          required_argument, 0, 'c'},
             {"version",        no_argument,       0, 'v'},
             {"help",           no_argument,       0, 'h'},
             {0,                0,                 0,  0 }
@@ -126,7 +185,7 @@ void parse_options(int argc, char *argv[])
         int option_index = 0;
 
         /* Parse argument using getopt_long */
-        c = getopt_long(argc, argv, "b:d:f:s:p:o:netl:m:vh", long_options, &option_index);
+        c = getopt_long(argc, argv, "b:d:f:s:p:o:net::Ll::m:c:vh", long_options, &option_index);
 
         /* Detect the end of the options */
         if (c == -1)
@@ -177,7 +236,11 @@ void parse_options(int argc, char *argv[])
                 break;
 
             case 't':
-                option.timestamp = true;
+                option.timestamp = timestamp_option_parse(optarg);
+                break;
+
+            case 'L':
+                option.list_devices = true;
                 break;
 
             case 'l':
@@ -189,9 +252,28 @@ void parse_options(int argc, char *argv[])
                 option.map = optarg;
                 break;
 
+            case 'c':
+                option.color = string_to_long(optarg);
+                if (option.color > 255)
+                {
+                    printf("Error: Invalid color code\n");
+                    exit(EXIT_FAILURE);
+                }
+                if (option.color < 0)
+                {
+                    // Print available color codes
+                    printf("Available color codes:\n");
+                    for (int i=0; i<=255; i++)
+                    {
+                        printf(" \e[1;38;5;%dmThis is color code %d\e[0m\n", i, i);
+                    }
+                    exit(EXIT_SUCCESS);
+                }
+                break;
+
             case 'v':
                 printf("tio v%s\n", VERSION);
-                printf("Copyright (c) 2014-2018 Martin Lund\n");
+                printf("Copyright (c) 2014-2022 Martin Lund\n");
                 printf("\n");
                 printf("License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>.\n");
                 printf("This is free software: you are free to change and redistribute it.\n");
@@ -214,20 +296,27 @@ void parse_options(int argc, char *argv[])
         }
     }
 
+    if (option.list_devices)
+    {
+        return;
+    }
+
     /* Assume first non-option is the tty device name */
-    if (optind < argc)
+    if (strcmp(option.tty_device, ""))
+	    optind++;
+    else if (optind < argc)
         option.tty_device = argv[optind++];
 
     if (strlen(option.tty_device) == 0)
     {
-        error_printf("Missing device name");
+        printf("Error: Missing device name\n");
         exit(EXIT_FAILURE);
     }
 
     /* Print any remaining command line arguments (unknown options) */
     if (optind < argc)
     {
-        printf("Error: unknown arguments: ");
+        printf("Error: Unknown argument ");
         while (optind < argc)
             printf("%s ", argv[optind++]);
         printf("\n");
